import sys

import yaml
from yamllint.linter import LintProblem

from Matcher import Matcher

ID = 'hardcoded'
TYPE = 'token'
CONF = {'password_re': str,
        'hostname_re': str}

def raise_problem(token, item_name):
    return LintProblem(token.start_mark.line, token.start_mark.column, 'Hardcoded %s' % item_name)


def check(conf, token, prev, next, nextnext, context):
    matcher = Matcher(None, conf['password_re'], conf['hostname_re'])
    if isinstance(token, yaml.tokens.ScalarToken):
        item = matcher.check(token, 'value', None)
        if item is not None:
            yield raise_problem(token, item) 







# -*- coding: utf-8 -*-
# Copyright (C) 2016 Peter Ericson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to forbid truthy values that are not quoted nor explicitly typed.

This would prevent YAML parsers from transforming ``[yes, FALSE, Off]`` into
``[true, false, false]`` or ``{y: 1, yes: 2, on: 3, true: 4, True: 5}`` into
``{y: 1, true: 5}``.

.. rubric:: Examples

#. With ``truthy: {}``

   the following code snippet would **PASS**:
   ::

    boolean: true

    object: {"True": 1, 1: "True"}

    "yes":  1
    "on":   2
    "true": 3
    "True": 4

     explicit:
       string1: !!str True
       string2: !!str yes
       string3: !!str off
       encoded: !!binary |
                  True
                  OFF
                  pad==  # this decodes as 'N\xbb\x9e8Qii'
       boolean1: !!bool true
       boolean2: !!bool "false"
       boolean3: !!bool FALSE
       boolean4: !!bool True
       boolean5: !!bool off
       boolean6: !!bool NO

   the following code snippet would **FAIL**:
   ::

    object: {True: 1, 1: True}

   the following code snippet would **FAIL**:
   ::

    yes:  1
    on:   2
    true: 3
    True: 4
"""

import yaml

from yamllint.linter import LintProblem

ID = 'truthy'
TYPE = 'token'
CONF = {}

TRUTHY = ['YES', 'Yes', 'yes',
          'NO', 'No', 'no',
          'TRUE', 'True',  # 'true' is a boolean
          'FALSE', 'False',  # 'false' is a boolean
          'ON', 'On', 'on',
          'OFF', 'Off', 'off']


def check(conf, token, prev, next, nextnext, context):
    if prev and isinstance(prev, yaml.tokens.TagToken):
        return

    if isinstance(token, yaml.tokens.ScalarToken):
        if token.value in TRUTHY and token.style is None:
            yield LintProblem(token.start_mark.line + 1,
                              token.start_mark.column + 1,
                              "truthy value is not quoted")



import re

class Matcher:
    ''' Incapsulates string constants matching for code static analysis. '''

    EMAIL_REGEX = '(^[a-zA-Z0-9._]+@[a-z0-9._]+$)|(^[^(%s)]@[^(%s)]$)'
    # match every IP except 127.0.0.* and 0.0.0.0
    IP_REGEX = '([0-9]{1,3}\.){3}(?<!127.0.0.)[0-9]{1,3}(?<!0.0.0.0)'

    email_re = re.compile(EMAIL_REGEX)
    ip_re = re.compile(IP_REGEX)

    items = ['hostname', 'ip', 'email', 'password']

    def __init__(self, caller, password_regex, hostname_regex):
        self.hostname_re = re.compile(hostname_regex)
        self.password_re = re.compile(password_regex)
        # checker instance
        self.caller = caller
        
    def _check(self, item, s):
        return eval('self.%s_re.search(s)' % item)

    def check(self, row, field, err_fn, *args):
        """
        Takes 'field' on 'row' and runs '_check'
        on hardcoded items from 'items' list.
        If check finds hardcoded it runs 'err_fn'
        which will report on lint error for specific linter
        whose instance is stored in 'self.caller'.
        """
        for item in self.items:
            val = getattr(row, field)
            if self._check(item, val):
                if self.caller is not None:
                    err_fn(self.caller, row, item, *args)
                else:
                    return item


